
#include "tank.h"
#include "driving.h"
#include "movement.h"
#include "simulation.h"



tank *tank_create(const point *p) {
    tank *t = NULL;
    t = malloc(sizeof(tank));
    tank_init(t);
    t->x = p->x;
    t->y = p->y;

    return t;
}


void tank_destroy(tank *t) {
    tank_movement_clear(t);
    tank_waypoint_clear(t);
    free(t);
}

void tank_copy(const tank *orig, tank *copy, bool withmovements, bool withwaypoints) {
    copy->x = orig->x;
    copy->y = orig->y;
//    copy->left_accel = orig->left_accel;
//    copy->right_accel = orig->right_accel;
    copy->a = orig->a;
    copy->b = orig->b;
    copy->r = orig->r;

    copy->a1 = orig->a1;
    copy->a2 = orig->a2;
    copy->b1 = orig->b1;
    copy->b2 = orig->b2;
    copy->hasolddata = orig->hasolddata;

    /* TODO: bool withmovements = true: moves auch übernehmen, dazu die liste kopieren (nicht einfach = setzen) */
    copy->moves = NULL;
    copy->waypoints = NULL;
    copy->idle = true;
    if(withmovements && orig->moves) {
        tank_movement_copy(orig, copy);
    }

    if(withwaypoints && orig->waypoints) {
        tank_waypoint_copy(orig, copy);
    }
}

void tank_init(tank *t) {
    t->x = 0;
    t->y = 0;
    t->r = 0;
//    t->left_accel = 0;
//    t->right_accel = 0;
    t->hasolddata = 2;
    t->a = 0;
    t->b = 0;
    t->idle = false;
    t->moves = NULL;
    t->waypoints = NULL;
}

void tank_loop(tank *t) {
    if(t->moves == NULL) {
        return;
    }
    if(t->idle) {
        if(t->moves->type == MT_MOVE) {
            t->idle = !tank_navigate(t, &t->moves->target);
        } else {
            t->idle = !tank_rotate(t, t->moves->r);
        }
        tank_drive(t);
    }
    if(!t->idle) {
        if(t->moves->type == MT_MOVE && t->waypoints) {
            tank_waypoint_drop(t);
        }
        tank_movement_drop(t);
        t->idle = true;
        t->hasolddata = 2;
    }
}

void tank_drive(tank *t) {
    float f;
    if(t->a == t->b) {
        /* go straight */
        t->x += cos(t->r)*t->a;
        t->y += sin(t->r)*t->a;
    } else {
        /* curve */
        f = differ(t->a, t->b)/WIDTH;
        if(f > M_PI_4)
            f = M_PI_4;

        t->r += asin(f) * (t->a > t->b ? 1 : -1);

        t->x += cos(t->r)*(t->a + t->b)/2;
        t->y += sin(t->r)*(t->a + t->b)/2;

        /* cosmetic */
        t->r = rotation_cosmetic(t->r);
    }
}

bool tank_navigate(tank *t, const point *target) {
    float u, v, ulen, tlen, speed;
    vector c, d, e;
    float f;

    c.x = cos(M_PI_2+t->r);
    c.y = sin(M_PI_2+t->r);

    d.x = cos(t->r);
    d.y = sin(t->r);

    e.x = (target->x - t->x);
    e.y = (target->y - t->y);

    u = -(d.x*e.y - e.x*d.y) / (c.x*d.y - d.x*c.y);
    v = (c.x*e.y - e.x*c.y) / (c.x*d.y - d.x*c.y);

    ulen = (u*c.x)*(u*c.x) + (u*c.y)*(u*c.y);
    //vlen = (v*d.x)*(v*d.x) + (v*d.y)*(v*d.y);
    tlen = (e.x)*(e.x) + (e.y)*(e.y);

    if(tlen < MAX_SPEED*MAX_SPEED) {
        if(tlen < WIDTH*WIDTH) {
            t->a = 0;
            t->b = 0;
            return true;
        } else {
            speed = MAX_SPEED;
        }
    } else {
        speed = MAX_SPEED;
    }

    f = 2*speed/tlen*ulen+2;
    t->a = (v > 0 ? 1 : -(MAX_REV_SPEED / MAX_SPEED)) * (u > 0 ? speed : speed - f);
    t->b = (v > 0 ? 1 : -(MAX_REV_SPEED / MAX_SPEED)) * (u > 0 ? speed - f : speed);

    //printf("x: %3.5f y: %3.5f a: %3.8f b: %3.8f u: %3.5f u1: %3.5f\n", t->x, t->y, t->a, t->b, u, t->u1);
    return false;
}

bool tank_rotate(tank *t, float rot) {
    float diff, dir, speed;

    rot = rotation_cosmetic(rot);

    dir = t->r > rot ? -1 : 1;

    diff = differ(t->r, rot);
    if(diff > M_PI) {
        diff = 2*M_PI - diff;
        dir = -dir;
    }

    if(diff < 0.001)
        return true;

    speed = (WIDTH/2)*diff;
    if(speed > MAX_REV_SPEED) {
        speed = MAX_REV_SPEED;
    }

    tank_set_speed(t, speed*dir, -speed*dir);
    return false;
}

void tank_set_speed(tank *t, float a, float b) {
    t->a2 = t->a1;
    t->a1 = t->a;
    t->a = a;

    t->b2 = t->b1;
    t->b1 = t->b;
    t->b = b;

    if(t->hasolddata > 0)
        t->hasolddata--;
}



