/*
    author: elio gubser
*/
#include <time.h>

#include "main.h"
#include "tank.h"

#include "driving.h"
#include "simulation.h"
#include "movement.h"

SDL_Surface *screen, *route;

bool init_sdl() {
    /* initialize SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        printf( "Unable to init SDL: %s\n", SDL_GetError() );
        return false;
    }
    atexit(SDL_Quit);
    screen = SDL_SetVideoMode(800, 600, 16,
                                           SDL_HWSURFACE|SDL_DOUBLEBUF);
    if ( !screen )
    {
        printf("Unable to set 800x600 video: %s\n", SDL_GetError());
        return false;
    }
    return true;
}

int main() {
    char buffer[256];

    tank *t = NULL;
    point p;
    bool done = false;
    bool pause = false;

    if(!init_sdl())
        return 1;

    route = SDL_CreateRGBSurface(SDL_HWSURFACE | SDL_RLEACCEL, screen->w, screen->h, 24, 0, 0, 0, 0);
    SDL_SetColorKey(route, SDL_SRCCOLORKEY, SDL_MapRGB(route->format, 0, 0, 0));
    p.x = 50;
    p.y = 50;
    t = tank_create(&p);


    /* event loop */
    while (!done)
    {
        SDL_Event event;
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
            case SDL_QUIT:
                done = true;
                break;
            case SDL_MOUSEBUTTONDOWN:
                p.x = event.button.x;
                p.y = event.button.y;
                //drive(t, &p);
                //tank_movement_move(t, &p);
                tank_waypoint_add(t, &p);
                tank_waypoint_recalc_movements(t);
                pause = false;
            case SDL_KEYDOWN:
                {
                    switch(event.key.keysym.sym) {
                        case SDLK_ESCAPE: done = true; break;
                        case SDLK_SPACE: pause = true; break;
                        default: break;
                    }
                    break;
                }
            case SDL_KEYUP:
                {
                    switch(event.key.keysym.sym) {
                        case SDLK_ESCAPE: done = true; break;
                        case SDLK_SPACE: pause = false; break;
                        default: break;
                    }
                    break;
                }
            }
        }

        tank_loop(t);


        if(pause) {
            continue;
        }

        /*if(t->x >= screen->w) {
            t->x = 0;
        } else if(t->x < 0) {
            t->x = screen->w-1;
        }
        if(t->y >= screen->h) {
            t->y = 0;
        } else if(t->y < 0) {
            t->y = screen->h-1;
        }*/

        point e, f;
        SDL_BlitSurface(route, NULL, screen, NULL);

        e.x = t->x + (WIDTH/2)*cos(M_PI_2+t->r);
        e.y = t->y + (WIDTH/2)*sin(M_PI_2+t->r);
        f.x = t->x - (WIDTH/2)*cos(M_PI_2+t->r);
        f.y = t->y - (WIDTH/2)*sin(M_PI_2+t->r);
        lineColor(screen, (int)e.x, (int)e.y, (int)f.x, (int)f.y, 0xFFFFFFFF);
        e.x = t->x + (WIDTH/4)*cos(t->r);
        e.y = t->y + (WIDTH/4)*sin(t->r);
        lineColor(screen, (int)e.x, (int)e.y, (int)t->x, (int)t->y, 0x00FF00FF);
        e.x = t->x + (WIDTH/4)*cos(M_PI+t->r);
        e.y = t->y + (WIDTH/4)*sin(M_PI+t->r);
        lineColor(screen, (int)e.x, (int)e.y, (int)t->x, (int)t->y, 0xFF0000FF);
        pixelColor(screen, (int)p.x, (int)p.y, 0xFFFFFFFF);
        pixelColor(screen, (int)t->x, (int)t->y, 0xFFFFFFFF);

        sprintf(buffer, "x: %3.2f, y: %3.2f, left track: %3.2f, right track: %3.2f, rotation: %3.2f", t->x, t->y, t->a, t->b, 180/(M_PI)*t->r);
        stringColor(screen, 10, screen->h - 100, buffer, 0xFFFFFFFF);

        SDL_Flip(screen);
        SDL_FillRect(screen, NULL, 0x00000000);
        SDL_Delay(10);
    }
    return 0;
}
