#include "tank.h"
#include "driving.h"
#include "movement.h"
#include "simulation.h"


void tank_movement_copy(const tank *orig, tank *copy) {
    movement *ptr = NULL, *cur = NULL;
    if(orig->moves) {
        ptr = orig->moves;
        while(ptr) {
            cur = tank_movement_add(copy);
            cur->type = ptr->type;
            cur->target = ptr->target;
            cur->r = ptr->r;

            ptr = ptr->next;
        }
    }
}

movement *tank_movement_add(tank *t) {
    movement *mov=NULL, *cur=NULL;

    mov =(movement*) malloc(sizeof(movement));

    if(t->moves) {
        cur = t->moves;
        while(cur) {
            if(cur->next == NULL)
                break;
            cur = cur->next;
        }

        cur->next = mov;
    } else {
        t->moves = mov;
        t->idle = true;
    }
    mov->next = NULL;
    return mov;
}

movement *tank_movement_rotate(tank *t, float r) {
    r = rotation_cosmetic(r);
    movement *mov=NULL;
    mov = tank_movement_add(t);
    mov->type = MT_ROTATE;
    mov->r = r;
    return mov;
}

movement *tank_movement_move(tank *t, point *target) {
    movement *mov=NULL;
    mov = tank_movement_add(t);
    mov->type = MT_MOVE;
    mov->target.x = target->x;
    mov->target.y = target->y;
    return mov;
}

void tank_movement_clear(tank *t) {
    movement *cur = NULL, *next=NULL;
    cur = t->moves;
    t->moves = NULL;
    while(cur) {
        next = cur->next;
        free(cur);
        cur = next;
    }
    t->idle = false;
}

void tank_movement_drop(tank *t) {
    movement *unqeued;

    if(t->moves == NULL)
        return;

    unqeued = t->moves;
    t->moves = t->moves->next;

    free(unqeued);
    return;
}

unsigned int tank_movement_chronometry(tank *t, bool useptr) {
    unsigned int u;
    point p;
    tank *dummy = NULL;

    if(useptr) {
        dummy = t;
    } else {
        p.x = t->x;
        p.y = t->y;
        dummy = tank_create(&p);
        tank_copy(t, dummy, true, false);
    }

    u = 0;
    while(dummy->moves) {
        tank_loop(t);
        pixelColor(route, dummy->x, dummy->y, 0x00FFFFFF);
        u++;
    }

    if(!useptr) {
        tank_destroy(dummy);
    }
    return u;
}
